REMOTE_MANIFEST_NAME = "mjthor_resource_file_to_size_mb.json"
COMBINED_TRIES_NAME = "mjthor_resources_combined_meta.json.gz"
LOCAL_MANIFEST_NAME = "mjthor_data_type_to_source_to_versions.json"

TQDM_DISABLE_THRES = 3
